@@ -17,7 +17,19 @@ module Sequel
     # Contains procs keyed on subadapter type that extend the
     # given database object so it supports the correct database type.
     DATABASE_SETUP = {}
+
+    # Create custom NativeException alias for nicer access, and also so that
+    # JRuby 9.2+ so it doesn't use the deprecated ::NativeException
+    NativeException = java.lang.Exception
     
+    # Default database error classes
+    DATABASE_ERROR_CLASSES = [NativeException]
+    if JRUBY_VERSION < '9.2'
+      # On JRuby <9.2, still include ::NativeException, as it is still needed in some cases
+      DATABASE_ERROR_CLASSES << ::NativeException
+    end
+    DATABASE_ERROR_CLASSES.freeze
+
     # Allow loading the necessary JDBC support via a gem.
     def self.load_gem(name)
       begin
@@ -39,55 +51,80 @@ module Sequel
     # Raise a Sequel::AdapterNotFound if evaluating the class name raises a NameError.
     def self.load_driver(drv, gem=nil)
       load_gem(gem) if gem
-      eval drv
+      if drv.is_a?(String)
+        eval drv
+      else
+        *try, last = drv
+        try.each do |try_drv|
+          begin
+            return eval(try_drv)
+          rescue NameError
+          end
+        end
+
+        eval last
+      end
     rescue NameError
       raise Sequel::AdapterNotFound, "#{drv} not loaded#{", try installing jdbc-#{gem.to_s.downcase} gem" if gem}"
     end
 
     class TypeConvertor
+      CONVERTORS = convertors = {}
       %w'Boolean Float Double Int Long Short'.each do |meth|
-        class_eval("def #{meth}(r, i) v = r.get#{meth}(i); v unless r.wasNull end", __FILE__, __LINE__)
+        x = x = convertors[meth.to_sym] = Object.new
+        class_eval("def x.call(r, i) v = r.get#{meth}(i); v unless r.wasNull end", __FILE__, __LINE__)
       end
       %w'Object Array String Time Date Timestamp BigDecimal Blob Bytes Clob'.each do |meth|
-        class_eval("def #{meth}(r, i) r.get#{meth}(i) end", __FILE__, __LINE__)
+        x = x = convertors[meth.to_sym] = Object.new
+        class_eval("def x.call(r, i) r.get#{meth}(i) end", __FILE__, __LINE__)
       end
-      def RubyTime(r, i)
+      x = convertors[:RubyTime] = Object.new
+      def x.call(r, i)
         if v = r.getTime(i)
           Sequel.string_to_time("#{v.to_string}.#{sprintf('%03i', v.getTime.divmod(1000).last)}")
         end
       end
-      def RubyDate(r, i)
+      x = convertors[:RubyDate] = Object.new
+      def x.call(r, i)
         if v = r.getDate(i)
           Date.civil(v.getYear + 1900, v.getMonth + 1, v.getDate)
         end
       end
-      def RubyTimestamp(r, i)
+      x = convertors[:RubyTimestamp] = Object.new
+      def x.call(r, i)
         if v = r.getTimestamp(i)
           Sequel.database_to_application_timestamp([v.getYear + 1900, v.getMonth + 1, v.getDate, v.getHours, v.getMinutes, v.getSeconds, v.getNanos])
         end
       end
-      def RubyBigDecimal(r, i)
+      x = convertors[:RubyBigDecimal] = Object.new
+      def x.call(r, i)
         if v = r.getBigDecimal(i)
-          BigDecimal.new(v.to_string)
+          ::Kernel::BigDecimal(v.to_string)
         end
       end
-      def RubyBlob(r, i)
+      x = convertors[:RubyBlob] = Object.new
+      def x.call(r, i)
         if v = r.getBytes(i)
           Sequel::SQL::Blob.new(String.from_java_bytes(v))
         end
       end
-      def RubyClob(r, i)
+      x = convertors[:RubyClob] = Object.new
+      def x.call(r, i)
         if v = r.getClob(i)
           v.getSubString(1, v.length)
         end
       end
+      x = convertors[:RubyArray] = Object.new
+      def x.call(r, i)
+        if v = r.getArray(i)
+          v.array.to_ary
+        end
+      end 
 
-      o = new
-      MAP = Hash.new(o.method(:Object))
+      MAP = Hash.new(convertors[:Object])
       types = Java::JavaSQL::Types
 
       {
-        :ARRAY => :Array,
         :BOOLEAN => :Boolean,
         :CHAR => :String,
         :DOUBLE => :Double,
@@ -101,11 +138,12 @@ module Sequel
         :TINYINT => :Short,
         :VARCHAR => :String,
       }.each do |type, meth|
-        MAP[types.const_get(type)] = o.method(meth) 
+        MAP[types.const_get(type)] = convertors[meth]
       end
       BASIC_MAP = MAP.dup
 
       {
+        :ARRAY => :Array,
         :BINARY => :Blob,
         :BLOB => :Blob,
         :CLOB => :Clob,
@@ -118,13 +156,11 @@ module Sequel
         :TIMESTAMP => :Timestamp,
         :VARBINARY => :Blob,
       }.each do |type, meth|
-        BASIC_MAP[types.const_get(type)] = o.method(meth) 
-        MAP[types.const_get(type)] = o.method(:"Ruby#{meth}") 
+        BASIC_MAP[types.const_get(type)] = convertors[meth]
+        MAP[types.const_get(type)] = convertors[:"Ruby#{meth}"]
       end
-
       MAP.freeze
       BASIC_MAP.freeze
-      freeze
     end
 
     class Database < Sequel::Database
@@ -168,7 +204,7 @@ module Sequel
                 last_insert_id(conn, opts)
               end
             end
-          rescue NativeException, JavaSQL::SQLException => e
+          rescue *DATABASE_ERROR_CLASSES => e
             raise_error(e)
           ensure
             cps.close
@@ -189,7 +225,7 @@ module Sequel
             JavaSQL::DriverManager.setLoginTimeout(opts[:login_timeout]) if opts[:login_timeout]
             raise StandardError, "skipping regular connection" if opts[:jdbc_properties]
             JavaSQL::DriverManager.getConnection(*args)
-          rescue JavaSQL::SQLException, NativeException, StandardError => e
+          rescue StandardError, *DATABASE_ERROR_CLASSES => e
             raise e unless driver
             # If the DriverManager can't get the connection - use the connect
             # method of the driver. (This happens under Tomcat for instance)
@@ -203,7 +239,7 @@ module Sequel
               c = driver.new.connect(args[0], props)
               raise(Sequel::DatabaseError, 'driver.new.connect returned nil: probably bad JDBC connection string') unless c
               c
-            rescue JavaSQL::SQLException, NativeException, StandardError => e2
+            rescue StandardError, *DATABASE_ERROR_CLASSES => e2
               if e2.respond_to?(:message=) && e2.message != e.message
                 e2.message = "#{e2.message}\n#{e.class.name}: #{e.message}"
               end
@@ -211,7 +247,7 @@ module Sequel
             end
           end
         end
-        setup_connection(conn)
+        setup_connection_with_opts(conn, opts)
       end
 
       # Close given adapter connections, and delete any related prepared statements.
@@ -236,7 +272,9 @@ module Sequel
                 log_connection_yield(sql, conn){stmt.execute(sql)}
               when :insert
                 log_connection_yield(sql, conn){execute_statement_insert(stmt, sql)}
-                last_insert_id(conn, Hash[opts].merge!(:stmt=>stmt))
+                opts = Hash[opts]
+                opts[:stmt] = stmt
+                last_insert_id(conn, opts)
               else
                 log_connection_yield(sql, conn){stmt.executeUpdate(sql)}
               end
@@ -355,7 +393,7 @@ module Sequel
       end
 
       def database_error_classes
-        [NativeException]
+        DATABASE_ERROR_CLASSES
       end
 
       def database_exception_sqlstate(exception, opts)
@@ -431,12 +469,15 @@ module Sequel
                 log_connection_yield(msg, conn, args){cps.execute}
               when :insert
                 log_connection_yield(msg, conn, args){execute_prepared_statement_insert(cps)}
-                last_insert_id(conn, Hash[opts].merge!(:prepared=>true, :stmt=>cps))
+                opts = Hash[opts]
+                opts[:prepared] = true
+                opts[:stmt] = cps
+                last_insert_id(conn, opts)
               else
                 log_connection_yield(msg, conn, args){cps.executeUpdate}
               end
             end
-          rescue NativeException, JavaSQL::SQLException => e
+          rescue *DATABASE_ERROR_CLASSES => e
             raise_error(e)
           ensure
             cps.close unless name
@@ -585,6 +626,11 @@ module Sequel
         conn
       end
 
+      # Setup the connection using the given connection options. Return the connection.  Can be overridden in subadapters for database specific setup.
+      def setup_connection_with_opts(conn, opts)
+        setup_connection(conn)
+      end
+
       def schema_column_set_db_type(schema)
         case schema[:type]
         when :string
@@ -656,7 +702,7 @@ module Sequel
       def statement(conn)
         stmt = conn.createStatement
         yield stmt
-      rescue NativeException, JavaSQL::SQLException => e
+      rescue *DATABASE_ERROR_CLASSES => e
         raise_error(e)
       ensure
         stmt.close if stmt
@@ -768,11 +814,14 @@ module Sequel
           i += 1
           cols << [output_identifier(meta.getColumnLabel(i)), i, convert ? type_convertor(map, meta, meta.getColumnType(i), i) : basic_type_convertor(map, meta, meta.getColumnType(i), i)]
         end
+        max = i
         self.columns = cols.map{|c| c[0]}
 
         while result.next
           row = {}
-          cols.each do |n, j, pr|
+          i = -1
+          while (i += 1) < max
+            n, j, pr = cols[i]
             row[n] = pr.call(result, j)
           end
           yield row
