@@ -14,10 +14,13 @@ Sequel.extension :core_extensions
 Sequel.extension :symbol_aref
 Sequel.extension :virtual_row_method_block
 
+ENV['MT_NO_PLUGINS'] = '1' # Work around stupid autoloading of plugins
 gem 'minitest'
-require 'minitest/autorun'
+require 'minitest/global_expectations/autorun'
 require 'minitest/hooks/default'
 
+require_relative "visibility_checking_after_hook" if ENV['CHECK_METHOD_VISIBILITY']
+
 require_relative "deprecation_helper.rb"
 
 describe "Sequel core extensions" do
@@ -26,6 +29,14 @@ describe "Sequel core extensions" do
   end
 end
 
+describe "Sequel.current" do
+  it "should be Thread.current unless fiber_concurrency extension is used" do
+    Sequel.current.must_equal Thread.current
+    Sequel.extension :fiber_concurrency
+    Sequel.current.must_equal Fiber.current
+  end
+end
+
 describe "Core extensions" do
   before do
     db = Sequel.mock
